package com.sapuseven.untis.ui.colorpicker

import androidx.compose.ui.geometry.Offset
import androidx.compose.ui.geometry.Size
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.graphics.drawscope.DrawScope
import androidx.compose.ui.graphics.drawscope.Stroke
import androidx.compose.ui.unit.dp

internal fun DrawScope.drawHorizontalSelector(amount: Float) {
	val halfIndicatorThickness = 4.dp.toPx()
	val strokeThickness = 1.dp.toPx()

	val offset =
		Offset(
			x = amount - halfIndicatorThickness,
			y = -strokeThickness
		)

	val selectionSize = Size(halfIndicatorThickness * 2f, this.size.height + strokeThickness * 2)
	drawSelectorIndicator(
		offset = offset,
		selectionSize = selectionSize,
		strokeThicknessPx = strokeThickness
	)
}

internal fun DrawScope.drawVerticalSelector(amount: Float) {
	val halfIndicatorThickness = 4.dp.toPx()
	val strokeThickness = 1.dp.toPx()

	val offset =
		Offset(
			y = amount - halfIndicatorThickness,
			x = -strokeThickness
		)
	val selectionSize = Size(this.size.width + strokeThickness * 2, halfIndicatorThickness * 2f)
	drawSelectorIndicator(
		offset = offset,
		selectionSize = selectionSize,
		strokeThicknessPx = strokeThickness
	)
}

internal fun DrawScope.drawSelectorIndicator(
	offset: Offset,
	selectionSize: Size,
	strokeThicknessPx: Float
) {
	val selectionStyle = Stroke(strokeThicknessPx)
	drawRect(
		Color.Gray,
		topLeft = offset,
		size = selectionSize,
		style = selectionStyle
	)
	drawRect(
		Color.White,
		topLeft = offset + Offset(strokeThicknessPx, strokeThicknessPx),
		size = selectionSize.inset(2 * strokeThicknessPx),
		style = selectionStyle
	)
}

internal fun Size.inset(amount: Float): Size {
	return Size(width - amount, height - amount)
}
